-- Power bar module

local powertype
local powcolor = {}
local predictBase = 14
local newPredict
local adj = 0
local spec

function SN:SetupPower()	

	local f = SN.F
	local m = SN.Media

	f.powercontainer = CreateFrame("Frame", nil, UIParent)
	f.powercontainer:SetPoint(unpack(SN.db.profile.Frames.power))
	if (IsAddOnLoaded("ElvUI")) then
		f.powercontainer:SetSize(250, 24)
		SN:SNCreateBG(f.powercontainer)
	else
		f.powercontainer:SetSize(258, 30)
		f.powercontainer:SetBackdrop({
		bgFile = m.media.blankTex, 
		edgeFile = m.media.blankTex, 
		tile = false, tileSize = 0, edgeSize = 4, 
		insets = { left = 4, right = 4, top = 4, bottom = 4 }
		})
		f.powercontainer:SetBackdropColor(0.1, 0.1, 0.1, 0.3)
		f.powercontainer:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
	end

	f.powercontainer.value = f.powercontainer:CreateFontString(nil, "ARTWORK")
	f.powercontainer.value:SetFont(m.media.font, 24, "OUTLINE")
	f.powercontainer.value:SetTextColor(0.78, 0.25, 0.25)
	f.powercontainer.value:SetText("")
	f.powercontainer.value:SetAllPoints(f.powercontainer)
	f.powercontainer.value:SetJustifyH("CENTER")
	f.powercontainer.value:SetJustifyV("MIDDLE")

	f.powercontainer.frameLocked = 1
	f.powercontainer.name = "powercontainer"
	--f.powercontainer:SetMovable(SN.db.profile.Frames.lock)
	--f.powercontainer:EnableMouse(true)
	f.powercontainer:SetScript("OnMouseDown", function(self, button)
		if self:IsMovable() then
			if button == "LeftButton" and not self.isMoving then
				self:StartMoving()
				self.isMoving = true
			end
		end
	end)
	f.powercontainer:SetScript("OnMouseUp", function(self, button)
		if self:IsMovable() then
			if button == "LeftButton" and self.isMoving then
				self:StopMovingOrSizing()
				self.isMoving = false
			end
		end
	end)

	--The actual power bar
	f.power = CreateFrame("StatusBar", nil, f.powercontainer)
	f.power:SetPoint("CENTER", f.powercontainer, "CENTER", 0, 0)
	f.power:SetSize(250, 23)
	f.power:SetStatusBarTexture(m.media.minTex)
	f.power:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
	f.power:SetMinMaxValues(0, (UnitPowerMax("player") > 0) and UnitPowerMax("player") or 100)	
	f.power:SetValue(UnitPower("player"))
	f.power.TimeSinceLastUpdate = 0
	f.power:SetScript("OnUpdate", UpdatePower)
	f.power:SetBackdrop({
				bgFile = m.media.blankTex,
			})
	f.power:SetBackdropColor(0.1, 0.1, 0.1, 0)
	f.power:SetScript("OnEvent", function(self, event, ...)
			local sourceName, spellName, _, _, spellId = ...
			
			if event == "UNIT_SPELLCAST_START" then
				if sourceName == "player" then						
					if spellId  == 56641 or spellId == 77767 then -- HUNTER: Steady Shot / Cobra Shot
						SN:updatePredict()
					end
				end
			elseif event == "UNIT_SPELLCAST_STOP" or event == "UNIT_SPELLCAST_FAILED" then
				if spellId == 56641 or spellId == 77767 then -- HUNTER: Steady Shot / Cobra Shot							
					self.predict:Hide()
				end
			elseif event == "PLAYER_SPECIALIZATION_CHANGED" then
				spec = GetSpecialization()
			end
		end)
	
	--Color time
	local class, classFileName, classID = UnitClass("player")
	local colortmp = RAID_CLASS_COLORS[classFileName]
	color = { colortmp.r, colortmp.g, colortmp.b }

	--The power text
	f.power.value = f.power:CreateFontString(nil, "OVERLAY")
	f.power.value:SetPoint("CENTER", f.power, "CENTER", 4, 0)
	f.power.value:SetFont(m.media.font, 14, "OUTLINE")
	f.power.value:SetJustifyH("CENTER")
	f.power.value:SetShadowOffset(1, -1)
	f.power.value:SetTextColor(unpack(color))
	f.power.value:SetText(UnitPower("player"))
	
	--SN:MakeSmooth(f.power)

	f.power.predict = CreateFrame("StatusBar", nil, f.power)			
	f.power.predict:SetStatusBarTexture(f.power:GetStatusBarTexture():GetTexture()) -- Use the main bar's texture
	f.power.predict:SetMinMaxValues(0, 1)
	f.power.predict:SetValue(1)
	f.power.predict:SetFrameLevel(f.power:GetFrameLevel())
	f.power.predict:SetSize((f.power:GetWidth() / 100) * (predictBase + SN:getPredictAdj()), f.power:GetHeight())

	-- Register Events to support the bar
	f.power:RegisterEvent("UNIT_SPELLCAST_START")
	f.power:RegisterEvent("UNIT_SPELLCAST_STOP")
	f.power:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	f.power:RegisterEvent("UNIT_SPELLCAST_FAILED")
	f.power:RegisterEvent("UNIT_MAXPOWER")
	f.power:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
	f.power:RegisterEvent("UNIT_POWER")

	if (select(2, UnitClass("player")) == "HUNTER") or (select(2, UnitClass("player")) == "DRUID" and GetSpecialization() == 2 or GetSpecialization() == 3)
					or (select(2, UnitClass("player")) == "WARRIOR") or (select(2, UnitClass("player")) == "ROGUE") then
		for i=1, 2 do
			if (GetClass(i, 1) == true) and (GetClass(i, 6) == 0)
				and (GetClass(1, 1) == true) then
				
				local _, _name, _cost
				if i > 1 then
					name, _, _, cost = GetSpellInfo(GetClass(i, 2))
				end
				
				if (i == 1) or ( (i > 1) and name and cost and (cost > 0) ) then
					SN.F.power["tick"..i] = CreateFrame("Frame", "SN_POWER_TICKS"..i, SN.F.power)
					SN.F.power["tick"..i]:SetSize(10, SN.F.power:GetHeight() * 1.6)
					SN.F.power["tick"..i].tex = SN.F.power["tick"..i]:CreateTexture(nil, "OVERLAY")
					SN.F.power["tick"..i].tex:SetAllPoints(SN.F.power["tick"..i])
					SN.F.power["tick"..i].tex:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")	
					SN.F.power["tick"..i].tex:SetBlendMode("ADD")
				end
				SN.F.power["tick"..i]:Hide()
			end
		end
	end

	SN:PowebarTicks()
	SN:GetPowerColor()

	if select(2, UnitPowerType("player")) == "RAGE" or select(2, UnitPowerType("player")) == "ENERGY" or 
		select(2, UnitPowerType("player")) == "FOCUS" then 

		f.powercontainer:Show()
	else
		f.powercontainer:Hide()
	end
end

function SN:GetPowerColor()
	
	powertype = select(2, UnitPowerType("player"))
	
	if powertype == "RAGE" then
		powcolor = SN.Media.power.RAGE
	elseif powertype == "FOCUS" then
		powcolor = SN.Media.power.FOCUS
	elseif powertype == "ENERGY" then
		powcolor = SN.Media.power.ENERGY
	elseif powertype == "RUNIC_POWER" then
		powcolor = SN.Media.power.RUNIC_POWER
	end
	
	return powcolor	
end

function UpdatePower(frame, elapsed)
	

	frame.TimeSinceLastUpdate = frame.TimeSinceLastUpdate + elapsed
	if frame.TimeSinceLastUpdate > SN_UpdateInterval then

		frame.TimeSinceLastUpdate = 0
		frame:SetValue(UnitPower("player"))
		frame:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
		frame.value:SetText(UnitPower("player"))
		SN.F.power:SetMinMaxValues(0, (UnitPowerMax("player") > 0) and UnitPowerMax("player") or 100)

		-- Spell tick marks
		if (select(2, UnitClass("player")) == "HUNTER") or (select(2, UnitClass("player")) == "DRUID" and GetSpecialization() == 2 or GetSpecialization() == 3)
					or (select(2, UnitClass("player")) == "WARRIOR") or (select(2, UnitClass("player")) == "ROGUE") then
				
			if SN.db.profile.powerbar.enable then
				local _, name, cost 
				for jj=1, 2 do
					if (GetClass(jj, 1) == true) and (frame["tick"..jj] ~= nil) 
						and (GetClass(1, 1) == true) then
				
						if (jj > 1) then
							name, _, _, cost = GetSpellInfo(GetClass(jj, 2))
						end
						if frame["tick"..jj]:IsShown() then
							if (jj == 1) or ( (jj > 1) and name and cost and (cost > 0) ) then
								if jj == 1 then
									frame["tick"..jj]:ClearAllPoints()
									frame["tick"..jj]:SetPoint("LEFT", frame, "LEFT", SN:GetSpecSpellCost() * (frame:GetWidth() / select(2, frame:GetMinMaxValues() ) ) - 5, 0)
								else
									frame["tick"..jj]:ClearAllPoints()
									frame["tick"..jj]:SetPoint("LEFT", frame, "LEFT", 
										( (GetClass(jj, 3) == true) and (SN:GetSpecSpellCost() + cost) or cost ) * 
										(frame:GetWidth() / select(2, frame:GetMinMaxValues() ) ) - 5, 0)
								end
							end
						end
					end
				end
			end
		end
	end
end

local function Smooth(self, value)
	if (value ~= self:GetValue()) or (value == 0) then
		self.smoothing = value
	else
		self.smoothing = nil
	end
end

function SN:MakeSmooth(powerframe)
	if powerframe.SetValue_ORI then return end
	powerframe.SetValue_ORI = powerframe.SetValue
	powerframe.SetValue = Smooth
	
	powerframe.smoother = CreateFrame("Frame", nil, powerframe)
	local min, max = math.min, math.max
	powerframe.smoother:SetScript("OnUpdate", function(self)
		local rate = GetFramerate()
		local limit = 30 / rate
		if self:GetParent().smoothing then
			local cur = self:GetParent():GetValue()
			local new = cur + min((self:GetParent().smoothing - cur) / 3, max(self:GetParent().smoothing - cur, limit))
			self:GetParent():SetValue_ORI(new)
			if cur == self:GetParent().smoothing or abs(new - self:GetParent().smoothing) < 2 then
				self:GetParent():SetValue_ORI(self:GetParent().smoothing)
				self:GetParent().smoothing = nil
			end
		end
	end)
end

function SN:RemoveSmooth(powerframe)
	if not powerframe.SetValue_ORI then return end
	
	powerframe.smoother:SetScript("OnUpdate", nil)
	powerframe.smoother:SetParent(nil)
	powerframe.smoother = nil
	
	powerframe.SetValue = powerframe.SetValue_ORI
	powerframe.SetValue_ORI = nil
end

function SN:PowebarTicks()
	local name, cost

	if (select(2, UnitClass("player")) == "HUNTER") or (select(2, UnitClass("player")) == "DRUID" and GetSpecialization() == 2 or GetSpecialization() == 3)
					or (select(2, UnitClass("player")) == "WARRIOR") or (select(2, UnitClass("player")) == "ROGUE") then
		
		for i=1, 2 do
			if (GetClass(i, 1) == true) and (GetClass(i, 6) == 0)
						and (GetClass(1, 1) == true) then
				
				if SN.db.profile.powerbar.enable then	
					local _, _name, _cost
					name, _, _, cost = GetSpellInfo(GetClass(i, 2))
					if (i == 1) or ( (i > 1) and name and cost and (cost > 0) ) then
						SN.F.power["tick"..i]:Show()
					end
				else
					if SN.F.power["tick"..i]:IsShown() then
						SN.F.power["tick"..i]:Hide()	
					end
				end
			end
		end
	end
end

function SN:GetSpecSpellCost()
	currentCost = select(4, GetSpellInfo(GetClass(1, 2)))

	if (currentCost ~= 0) then
		lastCost = currentCost
	end
	return lastCost
end

function SN:getPredictAdj()
	adj = 0
	
	if (GetSpecialization() == 2) and select(1, UnitAura("player", GetSpellInfo(53220), nil, "HELPFUL") ) then -- 53220 = Steady Focus (buff)
		adj = adj + 3
	end
	
	return (adj)
end

function SN:updatePredict()

	-- Add values if AotF active and talent points, and Termination points
	newPredict = predictBase + SN:getPredictAdj()
		
	SN.F.power.predict:SetSize((SN.F.power:GetWidth() / 100) * newPredict, SN.F.power:GetHeight() )

	SN.F.power.predict:ClearAllPoints()
	SN.F.power.predict:SetPoint("LEFT", SN.F.power, "LEFT",	SN.F.power:GetWidth() / (select(2, SN.F.power:GetMinMaxValues() ) / SN.F.power:GetValue() ), 0)

	if (UnitPower("player") + newPredict) > UnitPowerMax("player") then
		SN.F.power.predict:SetSize((SN.F.power:GetWidth() / 100) * (UnitPowerMax("player") - UnitPower("player") ), SN.F.power:GetHeight() )			
	end
		
	SN.F.power.predict:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
		
	if ( (UnitPowerMax("player") - UnitPower("player") ) > 0) and (not UnitIsDeadOrGhost("player") ) then
		SN.F.power.predict:SetAlpha(SN.F.power:GetAlpha() * 0.6)
		SN.F.power.predict:Show()
	else
		SN.F.power.predict:Hide()
	end
end

function SN:UpdatePowerType()
	-- setup the power bar
	if select(2, UnitPowerType("player")) == "RAGE" or select(2, UnitPowerType("player")) == "ENERGY" or 
		select(2, UnitPowerType("player")) == "FOCUS" then 

		SN:GetPowerColor()
		SN.F.power:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
		SN.F.power:SetMinMaxValues(0, (UnitPowerMax("player") > 0) and UnitPowerMax("player") or 100)

		SN.F.powercontainer:Show()
		if SN.db.profile.powerbar.enable then
			SN:PowebarTicks()
		end
	else
		SN.F.powercontainer:Hide()
	end
	
end

function GetClass(index, entry)
	local class, className = UnitClass("player")
	spec = GetSpecialization()

	if className == "HUNTER" then
		return SN.db.profile.powerbar.hunter[spec][index][entry]
	elseif className == "ROGUE" then
		return SN.db.profile.powerbar.rogue[spec][index][entry]
	elseif className == "DRUID" then
		if spec == 2 then
			spec = 1
		elseif spec == 3 then 
			spec = 2
		end
		return SN.db.profile.powerbar.druid[spec][index][entry]
	elseif className == "WARRIOR" then
		return SN.db.profile.powerbar.warrior[spec][index][entry]
	end
end